/*  xstartbutton - start another program on mouse click

    Copyright (C) 2006 Erik Auerswald <auerswal@unix-ag.uni-kl.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Command.h>

XtAppContext appContext;

static void usage(void)
{
  fprintf(stderr, "Usage: xstartbutton -program PROGRAM [-label LABEL]\n");
}

static void startProgram(Widget w, XtPointer client_data, XtPointer call_data)
{
  pid_t pid;
  int ret;

  w = call_data = NULL;
  pid = fork();
  if(pid > 0) { /* parent */
    return;
  }
  if(pid < 0) { /* error */
    fprintf(stderr, "xstartbutton: could not fork: %s\n", strerror(errno));
  }
  if(pid == 0) { /* child */
    ret = system((char *)client_data);
    exit(ret);
  }
}

int main (int argc, char **argv)
{
  Widget topLevel, startButton;
  char *label = NULL, *program = NULL;
  int i;

  topLevel = XtAppInitialize(&appContext, "XStartButton", NULL, 0, &argc,
                             argv, NULL, NULL, 0);


  for(i=1; i<argc; i++) {
    if(strcmp(argv[i], "-label") == 0) {
      label = argv[++i];
    } else if (strcmp(argv[i], "-program") == 0) {
      program = malloc(strlen(" exec") + strlen(argv[++i]) + 1);
      if (!program) {
        fprintf(stderr, "error: out of memory\n");
        exit(EXIT_FAILURE);
      }
      program = strcpy(program, "exec ");
      program = strcat(program, argv[i]);
    } else {
      fprintf(stderr, "error parsing command line: unknown option %s\n",
              argv[i]);
      usage();
      exit(EXIT_FAILURE);
    }
  }
  if(!program) {
    fprintf(stderr, "error: no -program argument\n");
    usage();
    exit(EXIT_FAILURE);
  }
  if(!label) {
    label = program+5;
  }

  signal(SIGCHLD, SIG_IGN); /* ignore status of childs */

  startButton = XtVaCreateManagedWidget("startButton", commandWidgetClass,
                                         topLevel, XtNlabel, label, NULL);
  XtAddCallback(startButton, XtNcallback, startProgram, (XtPointer) program);
  XtRealizeWidget(topLevel);
  XtAppMainLoop(appContext);
  exit(EXIT_SUCCESS);
}
